# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['opensr_test']

package_data = \
{'': ['*']}

install_requires = \
['kornia>=0.7.2',
 'lpips[perceptual]>=0.1.4',
 'matplotlib>=3.7.0',
 'mpltern>=1.0.4',
 'numpy>=1.25.2',
 'open-clip-torch[perceptual]>=2.24.0',
 'openai-clip[perceptual]>=1.0.1',
 'opencv-python>=4.8.0.0',
 'pydantic>=2.7.1',
 'requests>=2.25.0',
 'satalign>=0.1.3',
 'scikit-image>=0.19.0',
 'torchvision>=0.17.0']

setup_kwargs = {
    'name': 'opensr-test',
    'version': '1.2.0',
    'description': 'A comprehensive benchmark for real-world Sentinel-2 imagery super-resolution',
    'long_description': '<p align="center">\n  <a href="https://github.com/ESAOpenSR/opensr-test"><img src="docs/images/logo.png" alt="header" width="45%"></a>\n</p>\n\n<p align="center">\n    <em>\n    A comprehensive benchmark for real-world Sentinel-2 imagery super-resolution\n    </em>\n</p>\n\n<p align="center">\n<a href=\'https://pypi.python.org/pypi/opensr-test\'>\n<img src=\'https://img.shields.io/pypi/v/opensr-test.svg\' alt=\'PyPI\' />\n</a>\n<a href=\'https://colab.research.google.com/drive/1wDD_d0RUnAZkJTf63_t9qULjPjtCmbuv?usp=sharing\'>\n<img src=\'https://colab.research.google.com/assets/colab-badge.svg\' alt=\'COLAB\' />\n</a>\n<a href="https://opensource.org/licenses/MIT" target="_blank">\n    <img src="https://img.shields.io/badge/License-MIT-blue.svg" alt="License">\n</a>\n<a href="https://github.com/psf/black" target="_blank">\n    <img src="https://img.shields.io/badge/code%20style-black-000000.svg" alt="Black">\n</a>\n<a href="https://pycqa.github.io/isort/" target="_blank">\n    <img src="https://img.shields.io/badge/%20imports-isort-%231674b1?style=flat&labelColor=ef8336" alt="isort">\n</a>\n</p>\n\n---\n\n**GitHub**: [https://github.com/ESAOpenSR/opensr-test](https://github.com/ESAOpenSR/opensr-test)\n\n**Documentation**: [https://esaopensr.github.io/opensr-test](https://esaopensr.github.io/opensr-test)\n\n**PyPI**: [https://pypi.org/project/opensr-test/](https://pypi.org/project/opensr-test/)\n\n**Paper**: [https://ieeexplore.ieee.org/document/10530998](https://ieeexplore.ieee.org/document/10530998)\n\n---\n\n#\n\n## **Overview**\n\nSuper-Resolution (SR) aims to improve satellite imagery ground sampling distance. However, two problems are common in the literature. First, most models are **tested on synthetic data**, raising doubts about their real-world applicability and performance. Second, traditional evaluation metrics such as PSNR, LPIPS, and SSIM are not designed to assess SR performance. These metrics fall short, especially in conditions involving changes in luminance or spatial misalignments - scenarios frequently encountered in real world.\n\nTo address these challenges, \'opensr-test\' provides a fair approach for SR benchmark. We provide three datasets carefully crafted to minimize spatial and spectral misalignment. Besides, \'opensr-test\' precisely assesses SR algorithm performance across three independent metrics groups that measure consistency, synthesis, and correctness.\n\n<p align="center">\n  <img src="docs/images/diagram.png" alt="header">\n</p>\n\n## **How to use**\n\nThe example below shows how to use `opensr-test` to benchmark your SR model.\n\n\n```python\nimport torch\nimport opensr_test\n\nlr = torch.rand(4, 64, 64)\nhr = torch.rand(4, 256, 256)\nsr = torch.rand(4, 256, 256)\n\nmetrics = opensr_test.Metrics()\nmetrics.compute(lr=lr, sr=sr, hr=hr)\n>>> {\'reflectance\': 0.253, \'spectral\': 26.967, \'spatial\': 1.0, \'synthesis\': 0.2870, \'ha_percent\': 0.892, \'om_percent\': 0.0613, \'im_percent\': 0.04625}\n```\n\nThis model returns:\n\n- **reflectance**: How SR affects the reflectance norm of the LR image. By default, it uses the L1 distance. The lower the value, the better the reflectance consistency.\n\n- **spectral**: How SR affects the spectral signature of the LR image. By default, it uses the spectral angle distance (SAD). The lower the value, the better the spectral consistency. The angles are in degrees.\n\n- **spatial**: The spatial alignment between the SR and LR images. By default, it uses Phase Correlation Coefficient (PCC). Some SR models introduce spatial shift, which can be detected by this metric.\n\n- **synthesis**: The high-frequency details introduced by the SR model. By default, it uses the L1 distance. The higher the value, the better the synthesis quality.\n\n- **ha_metric**: The amount of hallucinations in the SR image. A hallucination is a detail (high-gradient) in the SR image that **is not present in the HR image.** The lower the value, the better the correctness of the SR image.\n\n- **om_metric**: The amount of omissions in the SR image. An omission is a detail in the HR image that **is not present in the SR image.** The lower the value, the better the correctness of the SR image.\n\n- **im_metric**: The amount of improvements in the SR image. An improvement is a detail in the SR image that **is present in the HR image and not in the LR image.** The higher the value, the better the correctness of the SR image.\n\n## **Benchmark**\n\nBenchmark comparison of SR models. Downward arrows (↓) denote metrics in which lower values are preferable, and upward arrows (↑) indicate that higher values reflect better performance. This table is an improved version of the one presented in the opensr-test paper, considering the latest version of the datasets and other distance metrics. To get the datasets used in the [original paper](https://ieeexplore.ieee.org/document/10530998), set the `dataset` parameter to `version=020` in the `opensr_test.load` function.\n\n### **CLIP**\n\nWe use the [**RemoteCLIP**](https://github.com/ChenDelong1999/RemoteCLIP) model to measure the distance between the LR, SR, and HR images. The parameters of the experiments are: `{"device": "cuda", "agg_method": "patch", "patch_size": 16, "correctness_distance": "clip"}`. This distance metric allows us to quantify the amount of semantic information introduced by the SR model. By comparing the three categories, we can assess the accuracy of the semantic information introduced.\n\n\n|    model     | reflectance ↓   | spectral ↓       | spatial ↓       | synthesis ↑     | ha_metric ↓     | om_metric ↓     | im_metric ↑     |\n|:-------------|:----------------|:-----------------|:----------------|:----------------|:----------------|:----------------|:----------------|\n| ldm_baseline | 0.1239 ± 0.0405 | 12.8441 ± 2.7508 | 0.0717 ± 0.0683 | 0.0409 ± 0.0290 | 0.5963 ± 0.3055 | 0.2327 ± 0.2238 | 0.1710 ± 0.1435 |\n| opensrmodel  | 0.0076 ± 0.0046 | 1.9739 ± 1.0507  | 0.0118 ± 0.0108 | 0.0194 ± 0.0120 | 0.1052 ± 0.0590 | 0.6927 ± 0.1343 | 0.2021 ± 0.0854 |\n| satlas       | 0.1197 ± 0.0233 | 15.1521 ± 2.9876 | 0.2766 ± 0.0741 | 0.0648 ± 0.0302 | 0.6996 ± 0.2058 | 0.0872 ± 0.0947 | 0.2132 ± 0.1393 |\n| sr4rs        | 0.0979 ± 0.0509 | 22.4905 ± 2.1168 | 1.0099 ± 0.0439 | 0.0509 ± 0.0237 | 0.3099 ± 0.1704 | 0.3486 ± 0.1753 | 0.3415 ± 0.1042 |\n| superimage   | 0.0068 ± 0.0016 | 1.8977 ± 1.1053  | 0.0004 ± 0.0032 | 0.0130 ± 0.0073 | 0.0610 ± 0.0305 | 0.8524 ± 0.0586 | 0.0866 ± 0.0395 |\n\n\n\n### **LPIPS**\n\nWe use the [**LPIPS**](https://github.com/richzhang/PerceptualSimilarity) model to measure the distance between the LR, SR, and HR images. The parameters of the experiments are: `{"device": "cuda", "agg_method": "patch", "patch_size": 16, "correctness_distance": "lpips"}`. This distance metric allows us to quantify the amount of perceptual information introduced by the SR model. By comparing the three categories, we can assess the accuracy of the perceptual information introduced.\n\n\n|    model     | reflectance ↓   | spectral ↓       | spatial ↓       | synthesis ↑     | ha_metric ↓     | om_metric ↓     | im_metric ↑     |\n|:-------------|:----------------|:-----------------|:----------------|:----------------|:----------------|:----------------|:----------------|\n| ldm_baseline | 0.1239 ± 0.0405 | 12.8441 ± 2.7508 | 0.0717 ± 0.0683 | 0.0409 ± 0.0290 | 0.4558 ± 0.2932 | 0.3558 ± 0.2518 | 0.1884 ± 0.1232 |\n| opensrmodel  | 0.0076 ± 0.0046 | 1.9739 ± 1.0507  | 0.0118 ± 0.0108 | 0.0194 ± 0.0120 | 0.0642 ± 0.0271 | 0.6690 ± 0.1291 | 0.2668 ± 0.1071 |\n| satlas       | 0.1197 ± 0.0233 | 15.1521 ± 2.9876 | 0.2766 ± 0.0741 | 0.0648 ± 0.0302 | 0.5999 ± 0.2182 | 0.0588 ± 0.0552 | 0.3413 ± 0.1858 |\n| sr4rs        | 0.0979 ± 0.0509 | 22.4905 ± 2.1168 | 1.0099 ± 0.0439 | 0.0509 ± 0.0237 | 0.3417 ± 0.1833 | 0.1924 ± 0.1402 | 0.4659 ± 0.1448 |\n| superimage   | 0.0068 ± 0.0016 | 1.8977 ± 1.1053  | 0.0004 ± 0.0032 | 0.0130 ± 0.0073 | 0.0357 ± 0.0200 | 0.8844 ± 0.0391 | 0.0800 ± 0.0301 |\n\n\n\n### **Normalized Difference (ND)**\n\nWe use the normalized difference (ND) distance to measure the distance between the LR, SR, and HR images. In contrast to L1 distance, ND is less sensitive to the magnitude of the reflectance values, providing a more fair detection of the hallucinations and omissions when assessing at the pixel level. The parameters of the experiments are: `{"device": "cuda", "agg_method": "patch", "patch_size": 1, "correctness_distance": "nd"}`. This distance metric allows us to quantify high-frequency details introduced by the SR model at pixel level. However, at pixel level, small changes in the reflectance values could introduce noise in the hallucination, omission, and improvement estimation.\n\n\n|    model     | reflectance ↓   | spectral ↓       | spatial ↓       | synthesis ↑     | ha_metric ↓     | om_metric ↓     | im_metric ↑     |\n|:-------------|:----------------|:-----------------|:----------------|:----------------|:----------------|:----------------|:----------------|\n| ldm_baseline | 0.0505 ± 0.0161 | 9.6923 ± 2.1742  | 0.0715 ± 0.0679 | 0.0285 ± 0.0307 | 0.6067 ± 0.2172 | 0.3088 ± 0.1786 | 0.0845 ± 0.0428 |\n| opensrmodel  | 0.0031 ± 0.0018 | 1.2632 ± 0.5878  | 0.0114 ± 0.0111 | 0.0068 ± 0.0044 | 0.3431 ± 0.0738 | 0.4593 ± 0.0781 | 0.1976 ± 0.0328 |\n| satlas       | 0.0489 ± 0.0086 | 12.1231 ± 3.1529 | 0.2742 ± 0.0748 | 0.0227 ± 0.0107 | 0.8004 ± 0.0641 | 0.1073 ± 0.0393 | 0.0923 ± 0.0266 |\n| sr4rs        | 0.0396 ± 0.0198 | 3.4044 ± 1.6882  | 1.0037 ± 0.1520 | 0.0177 ± 0.0083 | 0.7274 ± 0.0840 | 0.1637 ± 0.0572 | 0.1089 ± 0.0292 |\n| superimage   | 0.0029 ± 0.0009 | 1.5672 ± 1.0692  | 0.0132 ± 0.1131 | 0.0046 ± 0.0027 | 0.2026 ± 0.0692 | 0.6288 ± 0.0754 | 0.1686 ± 0.0302 |\n\n\nTo reproduce the results, check this [**Colab notebook**](https://colab.research.google.com/drive/1wDD_d0RUnAZkJTf63_t9qULjPjtCmbuv).\n\n## **Installation**\n\nInstall the latest version from PyPI:\n\n```\npip install opensr-test\npip install opensr-test[perceptual] # Install to test perceptual metrics\n```\n\nUpgrade `opensr-test` by running:\n\n```\npip install -U opensr-test\n```\n\nInstall the latest dev version from GitHub by running:\n\n```\npip install git+https://github.com/ESAOpenSR/opensr-test\n```\n\n## **Datasets**\n\nThe `opensr-test` package provides five datasets for benchmarking SR models. These datasets are carefully crafted to minimize spatial and spectral misalignment. See our Hugging Face repository for more details about the datasets. [**https://huggingface.co/datasets/isp-uv-es/opensr-test**](https://huggingface.co/datasets/isp-uv-es/opensr-test)\n\n| Dataset | Scale factor | Number of images | HR patch size |\n|---------|--------------|-------------------|--------------|\n| NAIP    | x4           | 62               | 484x484       |\n| SPOT    | x4           | 9                | 512x512       |\n| Venµs   | x2           | 59               | 256x256       |\n| SPAIN CROPS | x4       | 28               | 512x512       |\n| SPAIN URBAN | x4       | 20               | 512x512       |\n\n\n### **NAIP (X4 scale factor)**\n\nThe National Agriculture Imagery Program (NAIP) dataset is a high-resolution aerial imagery dataset that covers the continental United States. The dataset consists of 2.5m NAIP imagery captured in the visible and near-infrared spectrum (RGBNIR) and all Sentinel-2 L1C and L2A bands. The dataset focus in **crop fields, forests, and bare soil areas**.\n\n```python\nimport opensr_test\n\nnaip = opensr_test.load("naip")\n```\n\n<p align="center">\n  <a href="https://github.com/ESAOpenSR/opensr-test"><img src="docs/images/NAIP.gif" alt="header" width="80%"></a>\n</p>\n\n### **SPOT (X4 scale factor)**\n\nThe SPOT imagery were obtained from the worldstat dataset. The dataset consists of 2.5m SPOT imagery captured in the visible and near-infrared spectrum (RGBNIR) and all Sentinel-2 L1C and L2A bands. The dataset focus in **urban areas, crop fields, and bare soil areas**.\n\n```python\nimport opensr_test\n\nspot = opensr_test.load("spot")\n```\n\n<p align="center">\n  <a href="https://github.com/ESAOpenSR/opensr-test"><img src="docs/images/SPOT.gif" alt="header" width="80%"></a>\n</p>\n\n\n### **Venµs (X2 scale factor)**\n\nThe Venµs images were obtained from the [**Sen2Venµs dataset**](https://zenodo.org/records/6514159). The dataset consists of 5m Venµs imagery captured in the visible and near-infrared spectrum (RGBNIR) and all Sentinel-2 L1C and L2A bands. The dataset focus in **crop fields, forests, urban areas, and bare soil areas**.\n\n```python\nimport opensr_test\n\nvenus = opensr_test.load("venus")\n```\n\n<p align="center">\n  <a href="https://github.com/ESAOpenSR/opensr-test"><img src="docs/images/VENUS.gif" alt="header" width="80%"></a>\n</p>\n\n\n### **SPAIN CROPS (x4 scale factor)**\n\nThe SPAIN CROPS dataset consists of 2.5m aerial imagery captured in the visible and near-infrared spectrum (RGBNIR) by the Spanish National Geographic Institute (IGN). The dataset includes all Sentinel-2 L1C and L2A bands. The dataset focus in **crop fields and forests**.\n\n```python\nimport opensr_test\n\nspain_crops = opensr_test.load("spain_crops")\n```\n\n<p align="center">\n  <a href="https://github.com/ESAOpenSR/opensr-test"><img src="docs/images/SPAIN_CROPS.gif" alt="header" width="80%"></a>\n</p>\n\n\n### **SPAIN URBAN (x4 scale factor)**\n\nThe SPAIN URBAN dataset consists of 2.5m aerial imagery captured in the visible and near-infrared spectrum (RGBNIR) by the Spanish National Geographic Institute (IGN). The dataset includes all Sentinel-2 L1C and L2A bands. The dataset focus in **urban areas**.\n\n```python\n\nspain_urban = opensr_test.load("spain_urban")\n```\n\n<p align="center">\n  <a href="https://github.com/ESAOpenSR/opensr-test"><img src="docs/images/SPAIN_URBAN.gif" alt="header" width="80%"></a>\n</p>\n\n\n## **Examples**\n\nThe following examples show how to use `opensr-test` to benchmark SR models.\n\n| Model | Framework | Link |\n|-------|-----------|------|\n| SR4RS | TensorFlow | [![Model on HF](https://huggingface.co/datasets/huggingface/badges/resolve/main/model-on-hf-md.svg)](https://huggingface.co/isp-uv-es/superIX/blob/main/sr4rs/run.py) |\n| SuperImage | PyTorch | [![Model on HF](https://huggingface.co/datasets/huggingface/badges/resolve/main/model-on-hf-md.svg)](https://huggingface.co/isp-uv-es/superIX/blob/main/superimage/run.py) |\n| LDMSuperResolutionPipeline | Diffuser | [![Model on HF](https://huggingface.co/datasets/huggingface/badges/resolve/main/model-on-hf-md.svg)](https://huggingface.co/isp-uv-es/superIX/blob/main/ldm_baseline/run.py) |\n| opensr-model | Pytorch | [![Model on HF](https://huggingface.co/datasets/huggingface/badges/resolve/main/model-on-hf-md.svg)](https://huggingface.co/isp-uv-es/superIX/blob/main/opensrmodel/run.py) |\n| EvoLand | Pytorch | [![Model on HF](https://huggingface.co/datasets/huggingface/badges/resolve/main/model-on-hf-md.svg)](https://huggingface.co/isp-uv-es/superIX/blob/main/evoland/run.py) |\n| SWIN2-MOSE | Pytorch | [![Model on HF](https://huggingface.co/datasets/huggingface/badges/resolve/main/model-on-hf-md.svg)](https://huggingface.co/isp-uv-es/superIX/blob/main/swin2_mose/run.py) |\n\n## **Visualizations**\n\nThe `opensr-test` package provides a set of visualizations to help you understand the performance of your SR model.\n\n```python\nimport torch\nimport opensr_test\nimport matplotlib.pyplot as plt\n\nfrom super_image import HanModel\n\n# Define the SR model\nsrmodel = HanModel.from_pretrained(\'eugenesiow/han\', scale=4)\nsrmodel.eval()\n\n# Load the data\ndataset = opensr_test.load("spot")\nlr, hr = dataset["L2A"], dataset["HRharm"]\n\n# Define the benchmark experiment\nconfig = opensr_test.Config()\nmetrics = opensr_test.Metrics(config=config)\n\n# Define the image to be tested\nidx = 0\nlr_img = torch.from_numpy(lr[idx, 0:3]) / 10000\nhr_img = torch.from_numpy(hr[idx, 0:3]) / 10000\nwith torch.no_grad():\n    sr_img = srmodel(lr_img[None]).squeeze()\n\n# Compute the metrics\nmetrics.compute(\n    lr=lr_img, sr=sr_img, hr=hr_img\n)\n```\n\nNow, we can visualize the results using the `opensr_test.plot_*` module. To display the triplets LR, SR and HR images:\n\n```python\nmetrics.plot_triplets()\n```\n\n<p align="center">\n  <img src="docs/images/img_01.gif">\n</p>\n\nDisplay the summary of the metrics. The plot shows:\n  - LR: Low Resolution image \n  - LRdown: Downsampled Low Resolution image using bilinear interpolation and triangular antialiasing filter.\n  - SR: Super-Resolved image.\n  - SRharm: Harmonized super-resolution image.\n  - HR: High Resolution image.\n  - Reflectance Consistency: Reflectance consistency between the LR and HR images.\n  - Spectral Consistency: Spectral consistency between the LR and HR images.\n  - Distance normalized to the Omissiom, Hallucination, and Improvement spaces.\n\n```python\nmetrics.plot_summary()\n```\n\n<p align="center">\n  <img src="docs/images/img_02.gif">\n</p>\n\n\nDisplay the correctness of the SR image. The blue color represents the pixels closer to the improvement space (HR), green pixels are closer to the omission space (LR), and red pixels are closer to the hallucination space (neither in HR nor LR). The distance SR-LR and SR-HR are normalized to the LR-HR distance that is independent of the SR model. Threfore a pixel or patch with a improvement distance ($d_{im}$) of 3 means that the SR is further to the HR in 3 LR-HR units. The same applies to the omission distance.\n\n```python\nmetrics.plot_tc()\n```\n\n<p align="center">\n  <img src="docs/images/img_03.gif">\n</p>\n\n\nDisplay the histogram of the distances before and after the normalization:\n\n```python\nmetrics.plot_stats()\n```\n\n<p align="center">\n  <img src="docs/images/img_04.gif">\n</p>\n\nDisplay a ternary plot of the metrics:\n\n```python\nmetrics.plot_ternary()\n```\n\n<p align="center">\n  <img src="docs/images/img_05.gif">\n</p>\n\n## **Deeper understanding**\n\nExplore the [**API**](https://esaopensr.github.io/opensr-test/docs/API/config_pydantic.html) section for more details about personalizing your benchmark experiments.\n\n## **Citation**\n\nIf you use `opensr-test` in your research, please cite our paper:\n\n```\n@ARTICLE{10530998,\n  author={Aybar, Cesar and Montero, David and Donike, Simon and Kalaitzis, Freddie and Gómez-Chova, Luis},\n  journal={IEEE Geoscience and Remote Sensing Letters}, \n  title={A Comprehensive Benchmark for Optical Remote Sensing Image Super-Resolution}, \n  year={2024},\n  volume={21},\n  number={},\n  pages={1-5},\n  keywords={Measurement;Remote sensing;Spatial resolution;Superresolution;Reflectivity;Protocols;Inspection;Benchmarking;datasets;deep learning;NAIP;Sentinel-2 (S2);SPOT;super-resolution (SR)},\n  doi={10.1109/LGRS.2024.3401394}}\n```\n\n## **Acknowledgements**\n\nThis work was make with the support of the European Space Agency (ESA) under the project “Explainable AI: application to trustworthy super-resolution (OpenSR)”. Cesar Aybar acknowledges support by the National Council of Science, Technology, and Technological Innovation (CONCYTEC, Peru) through the “PROYECTOS DE INVESTIGACIÓN BÁSICA – 2023-01” program with contract number PE501083135-2023-PROCIENCIA. Luis Gómez-Chova acknowledges support from the Spanish Ministry of Science and Innovation (project PID2019-109026RB-I00 funded by MCIN/AEI/10.13039/501100011033).\n',
    'author': 'Cesar Aybar',
    'author_email': 'cesar.aybar@uv.es',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/csaybar/opensr-test',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)
